obj/conf_771254cb95770aaa6f3972567de8a47d/static/zstd_decompress.o: \
 decompress/zstd_decompress.c decompress/../common/zstd_deps.h \
 decompress/../common/mem.h decompress/../common/compiler.h \
 decompress/../common/portability_macros.h decompress/../common/debug.h \
 decompress/../common/zstd_deps.h decompress/../common/fse.h \
 decompress/../common/bitstream.h decompress/../common/mem.h \
 decompress/../common/error_private.h \
 decompress/../common/../zstd_errors.h decompress/../common/bits.h \
 decompress/../common/huf.h decompress/../common/fse.h \
 decompress/../common/xxhash.h decompress/../common/zstd_internal.h \
 decompress/../common/cpu.h decompress/../common/../zstd.h \
 decompress/../common/huf.h decompress/../common/xxhash.h \
 decompress/../common/zstd_trace.h decompress/zstd_decompress_internal.h \
 decompress/zstd_ddict.h decompress/../zstd.h \
 decompress/zstd_decompress_block.h decompress/../common/bits.h
decompress/../common/zstd_deps.h:
decompress/../common/mem.h:
decompress/../common/compiler.h:
decompress/../common/portability_macros.h:
decompress/../common/debug.h:
decompress/../common/zstd_deps.h:
decompress/../common/fse.h:
decompress/../common/bitstream.h:
decompress/../common/mem.h:
decompress/../common/error_private.h:
decompress/../common/../zstd_errors.h:
decompress/../common/bits.h:
decompress/../common/huf.h:
decompress/../common/fse.h:
decompress/../common/xxhash.h:
decompress/../common/zstd_internal.h:
decompress/../common/cpu.h:
decompress/../common/../zstd.h:
decompress/../common/huf.h:
decompress/../common/xxhash.h:
decompress/../common/zstd_trace.h:
decompress/zstd_decompress_internal.h:
decompress/zstd_ddict.h:
decompress/../zstd.h:
decompress/zstd_decompress_block.h:
decompress/../common/bits.h:
