#ifndef PQCLEAN_SNTRUP761_CLEAN_CRYPTO_CORE_WEIGHTSNTRUP761_H
#define PQCLEAN_SNTRUP761_CLEAN_CRYPTO_CORE_WEIGHTSNTRUP761_H

#include <stdint.h>
#define PQCLEAN_SNTRUP761_CLEAN_crypto_core_weightsntrup761_OUTPUTBYTES 2
#define PQCLEAN_SNTRUP761_CLEAN_crypto_core_weightsntrup761_INPUTBYTES 761
#define PQCLEAN_SNTRUP761_CLEAN_crypto_core_weightsntrup761_KEYBYTES 0
#define PQCLEAN_SNTRUP761_CLEAN_crypto_core_weightsntrup761_CONSTBYTES 0

int PQCLEAN_SNTRUP761_CLEAN_crypto_core_weightsntrup761(unsigned char *outbytes, const unsigned char *inbytes);
#endif
