obj/conf_771254cb95770aaa6f3972567de8a47d/static/fse_compress.o: \
 compress/fse_compress.c compress/../common/compiler.h \
 compress/../common/portability_macros.h compress/../common/mem.h \
 compress/../common/compiler.h compress/../common/debug.h \
 compress/../common/zstd_deps.h compress/../common/debug.h \
 compress/hist.h compress/../common/zstd_deps.h \
 compress/../common/bitstream.h compress/../common/mem.h \
 compress/../common/error_private.h compress/../common/../zstd_errors.h \
 compress/../common/bits.h compress/../common/fse.h \
 compress/../common/bitstream.h compress/../common/error_private.h \
 compress/../common/bits.h
compress/../common/compiler.h:
compress/../common/portability_macros.h:
compress/../common/mem.h:
compress/../common/compiler.h:
compress/../common/debug.h:
compress/../common/zstd_deps.h:
compress/../common/debug.h:
compress/hist.h:
compress/../common/zstd_deps.h:
compress/../common/bitstream.h:
compress/../common/mem.h:
compress/../common/error_private.h:
compress/../common/../zstd_errors.h:
compress/../common/bits.h:
compress/../common/fse.h:
compress/../common/bitstream.h:
compress/../common/error_private.h:
compress/../common/bits.h:
