obj/conf_771254cb95770aaa6f3972567de8a47d/static/error_private.o: \
 common/error_private.c common/error_private.h common/../zstd_errors.h \
 common/compiler.h common/portability_macros.h common/debug.h \
 common/zstd_deps.h
common/error_private.h:
common/../zstd_errors.h:
common/compiler.h:
common/portability_macros.h:
common/debug.h:
common/zstd_deps.h:
