obj/conf_771254cb95770aaa6f3972567de8a47d/static/pool.o: common/pool.c \
 common/zstd_deps.h common/debug.h common/zstd_internal.h \
 common/compiler.h common/portability_macros.h common/cpu.h common/mem.h \
 common/error_private.h common/../zstd_errors.h common/../zstd.h \
 common/fse.h common/bitstream.h common/bits.h common/huf.h \
 common/xxhash.h common/zstd_trace.h common/pool.h
common/zstd_deps.h:
common/debug.h:
common/zstd_internal.h:
common/compiler.h:
common/portability_macros.h:
common/cpu.h:
common/mem.h:
common/error_private.h:
common/../zstd_errors.h:
common/../zstd.h:
common/fse.h:
common/bitstream.h:
common/bits.h:
common/huf.h:
common/xxhash.h:
common/zstd_trace.h:
common/pool.h:
