obj/conf_771254cb95770aaa6f3972567de8a47d/static/hist.o: compress/hist.c \
 compress/../common/mem.h compress/../common/compiler.h \
 compress/../common/portability_macros.h compress/../common/debug.h \
 compress/../common/zstd_deps.h compress/../common/debug.h \
 compress/../common/error_private.h compress/../common/../zstd_errors.h \
 compress/hist.h compress/../common/zstd_deps.h
compress/../common/mem.h:
compress/../common/compiler.h:
compress/../common/portability_macros.h:
compress/../common/debug.h:
compress/../common/zstd_deps.h:
compress/../common/debug.h:
compress/../common/error_private.h:
compress/../common/../zstd_errors.h:
compress/hist.h:
compress/../common/zstd_deps.h:
