obj/conf_771254cb95770aaa6f3972567de8a47d/static/zstd_opt.o: \
 compress/zstd_opt.c compress/zstd_compress_internal.h \
 compress/../common/zstd_internal.h compress/../common/compiler.h \
 compress/../common/portability_macros.h compress/../common/cpu.h \
 compress/../common/mem.h compress/../common/debug.h \
 compress/../common/zstd_deps.h compress/../common/error_private.h \
 compress/../common/../zstd_errors.h compress/../common/../zstd.h \
 compress/../common/fse.h compress/../common/bitstream.h \
 compress/../common/bits.h compress/../common/huf.h \
 compress/../common/xxhash.h compress/../common/zstd_trace.h \
 compress/zstd_cwksp.h compress/../common/bits.h compress/hist.h \
 compress/../common/zstd_deps.h compress/zstd_opt.h
compress/zstd_compress_internal.h:
compress/../common/zstd_internal.h:
compress/../common/compiler.h:
compress/../common/portability_macros.h:
compress/../common/cpu.h:
compress/../common/mem.h:
compress/../common/debug.h:
compress/../common/zstd_deps.h:
compress/../common/error_private.h:
compress/../common/../zstd_errors.h:
compress/../common/../zstd.h:
compress/../common/fse.h:
compress/../common/bitstream.h:
compress/../common/bits.h:
compress/../common/huf.h:
compress/../common/xxhash.h:
compress/../common/zstd_trace.h:
compress/zstd_cwksp.h:
compress/../common/bits.h:
compress/hist.h:
compress/../common/zstd_deps.h:
compress/zstd_opt.h:
