obj/conf_771254cb95770aaa6f3972567de8a47d/static/entropy_common.o: \
 common/entropy_common.c common/mem.h common/compiler.h \
 common/portability_macros.h common/debug.h common/zstd_deps.h \
 common/error_private.h common/../zstd_errors.h common/fse.h \
 common/bitstream.h common/bits.h common/huf.h
common/mem.h:
common/compiler.h:
common/portability_macros.h:
common/debug.h:
common/zstd_deps.h:
common/error_private.h:
common/../zstd_errors.h:
common/fse.h:
common/bitstream.h:
common/bits.h:
common/huf.h:
