/*
	Licensed to the Apache Software Foundation (ASF) under one
	or more contributor license agreements.  See the NOTICE file
	distributed with this work for additional information
	regarding copyright ownership.  The ASF licenses this file
	to you under the Apache License, Version 2.0 (the
	"License"); you may not use this file except in compliance
	with the License.  You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing,
	software distributed under the License is distributed on an
	"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
	KIND, either express or implied.  See the License for the
	specific language governing permissions and limitations
	under the License.
*/

#ifndef CONFIG_FIELD_BLS381_H
#define CONFIG_FIELD_BLS381_H

#include"amcl.h"
#include "config_big_384_29.h"

// FP stuff

#define MBITS_BLS381 381
#define MOD8_BLS381 3
#define MODTYPE_BLS381 NOT_SPECIAL
#define MAXXES_BLS381 25


#endif
